@@ -4,6 +4,7 @@
 #include <string.h>
 #include <sys/time.h>
 
+#include <ge-support.h>
 #include "clearlooks_style.h"
 #include "clearlooks_rc_style.h"
 #include "clearlooks_draw.h"
@@ -11,8 +12,6 @@
 
 /* #define DEBUG 1 */
 
-#define HAVE_ANIMATION 1
-
 #define SCALE_SIZE 5
 
 #define DETAIL(xx)   ((detail) && (!strcmp(xx, detail)))
@@ -31,7 +30,7 @@
                      gint            height
 
 #ifdef HAVE_ANIMATION
-#include "animation.c"
+#include "animation.h"
 #endif
 
 static GtkStyleClass *parent_class;
@@ -45,7 +44,7 @@ clearlooks_begin_paint (GdkDrawable  *window, GdkRectangle *area)
 
     if (area) {
         cairo_rectangle (cr, area->x, area->y, area->width, area->height);
-        cairo_clip (cr);
+        cairo_clip_preserve (cr);
         cairo_new_path (cr);
     }
 
@@ -66,7 +65,7 @@ clearlooks_set_widget_parameters (const GtkWidget      *widget,
 	params->disabled   = (state_type == GTK_STATE_INSENSITIVE);			
 	params->state_type = (ClearlooksStateType)state_type;
 	params->corners    = CL_CORNER_ALL;
-		
+	params->rtl        = (cl_get_parent_direction (widget) == GTK_TEXT_DIR_RTL);
 	params->focus      = widget && GTK_WIDGET_HAS_FOCUS (widget);
 	params->is_default = widget && GTK_WIDGET_HAS_DEFAULT (widget);
 		
@@ -85,10 +84,57 @@ clearlooks_set_widget_parameters (const GtkWidget      *widget,
 static void
 draw_flat_box (DRAW_ARGS)
 {
-	parent_class->draw_flat_box (style, window, state_type,
-	                             shadow_type,
-	                             area, widget, detail,
-	                             x, y, width, height);
+	CHECK_ARGS
+	SANITIZE_SIZE
+	
+	if (detail && 	
+	    state_type == GTK_STATE_SELECTED && (
+	    !strncmp ("cell_even", detail, 9) ||
+	    !strncmp ("cell_odd", detail, 8)))
+	{
+		ClearlooksStyle  *clearlooks_style = CLEARLOOKS_STYLE (style);
+		ClearlooksColors *colors = &clearlooks_style->colors;
+		cairo_t          *cr = clearlooks_begin_paint (window, area);
+
+		cairo_pattern_t *pattern;
+		CairoColor       lower_color;
+		CairoColor       upper_color;
+
+		cairo_translate (cr, x, y);
+
+		if (GTK_WIDGET_HAS_FOCUS (widget))
+		{
+			upper_color = colors->base[state_type];
+		}
+		else
+		{
+			upper_color = colors->base[GTK_STATE_ACTIVE];
+		}
+
+		shade (&upper_color, &lower_color, 0.8);
+
+		pattern = cairo_pattern_create_linear (0, 0, 0, height);
+		cairo_pattern_add_color_stop_rgb (pattern, 0.0, upper_color.r,
+		                                                upper_color.g,
+		                                                upper_color.b);
+		cairo_pattern_add_color_stop_rgb (pattern, 1.0, lower_color.r,
+		                                                lower_color.g,
+		                                                lower_color.b);
+
+		cairo_set_source (cr, pattern);
+		cairo_rectangle  (cr, 0, 0, width, height);
+		cairo_fill       (cr);
+
+		cairo_pattern_destroy (pattern);
+		cairo_destroy (cr);
+	}
+	else
+	{
+		parent_class->draw_flat_box (style, window, state_type,
+		                             shadow_type,
+		                             area, widget, detail,
+		                             x, y, width, height);
+	}
 }
 
 static void
@@ -257,11 +303,15 @@ draw_handle (DRAW_ARGS, GtkOrientation orientation)
 	ClearlooksStyle  *clearlooks_style = CLEARLOOKS_STYLE (style);
 	ClearlooksColors *colors = &clearlooks_style->colors;
 	cairo_t          *cr;
+	gboolean         is_horizontal;
 	
 	sanitize_size (window, &width, &height);
 	
 	cr = clearlooks_begin_paint (window, area);
 	
+	// Evil hack to work around broken orientation for toolbars
+	is_horizontal = (width > height);
+	
 	if (DETAIL ("handlebox"))
 	{
 		WidgetParameters params;
@@ -269,8 +319,9 @@ draw_handle (DRAW_ARGS, GtkOrientation orientation)
 
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 		handle.type = CL_HANDLE_TOOLBAR;
-		handle.horizontal = FALSE;
+		handle.horizontal = is_horizontal;
 		
+		// Is this ever true? -Daniel
 		if (GTK_IS_TOOLBAR (widget) && shadow_type != GTK_SHADOW_NONE)
 		{
 			cairo_save (cr);
@@ -301,8 +352,9 @@ draw_handle (DRAW_ARGS, GtkOrientation orientation)
 
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 		handle.type = CL_HANDLE_TOOLBAR;
-		handle.horizontal = FALSE;
+		handle.horizontal = is_horizontal;
 		
+		// Is this ever true? -Daniel
 		if (GTK_IS_TOOLBAR (widget) && shadow_type != GTK_SHADOW_NONE)
 		{
 			cairo_save (cr);
@@ -324,15 +376,20 @@ draw_handle (DRAW_ARGS, GtkOrientation orientation)
 static int
 combo_box_get_seperator_pos (GtkWidget *widget)
 {
-	GList *children;
+	GList *children, *children_start;
+	GtkWidget *child;
 	int pos = 0;
 	
 	if (widget && GTK_IS_COMBO_BOX (widget->parent))
 	{
-		children = gtk_container_get_children (GTK_CONTAINER (widget));
+		children = children_start = gtk_container_get_children (GTK_CONTAINER (widget));
 		
 		if (children && GTK_IS_HBOX (children->data))
-			children = gtk_container_get_children (GTK_CONTAINER (children->data));
+		{
+			child = GTK_WIDGET (children->data);
+			g_list_free (children_start);
+			children = children_start = gtk_container_get_children (GTK_CONTAINER (child));
+		}
 
 		if (children)
 		{
@@ -347,6 +404,8 @@ combo_box_get_seperator_pos (GtkWidget *widget)
 	
 	pos += 2; /* don't ask me why.... widget->style->xthickness - 1 ? */
 	
+	g_list_free (children_start);	
+
 	return pos;
 }
 
@@ -367,13 +426,17 @@ draw_box (DRAW_ARGS)
 	else if (height == -1)
 		gdk_window_get_size (window, NULL, &height);
 
-	if (DETAIL ("menubar") && !(widget && cl_is_gnome_panel (widget->parent)))
+	if (DETAIL ("menubar") && 
+	    !(widget && (cl_is_panel_widget (widget->parent))))
 	{
 		WidgetParameters params;
+		MenuBarParameters menubar;
 		
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 
-		clearlooks_draw_menubar (cr, colors, &params,
+		menubar.style = clearlooks_style->menubarstyle;
+
+		clearlooks_draw_menubar (cr, colors, &params, &menubar,
 		                         x, y, width, height);
 	}
 	else if (DETAIL ("button") && widget && widget->parent &&
@@ -516,14 +579,17 @@ draw_box (DRAW_ARGS)
 		GtkAdjustment *adjustment = gtk_range_get_adjustment (GTK_RANGE (widget));	
 		WidgetParameters params;
 		SliderParameters slider;
+		gint slider_length;
 		
+		gtk_widget_style_get (widget, "slider-length", &slider_length, NULL);
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 		params.corners    = CL_CORNER_NONE;
 		
 		slider.inverted   = gtk_range_get_inverted (GTK_RANGE (widget));
 		slider.horizontal = (GTK_RANGE (widget)->orientation == GTK_ORIENTATION_HORIZONTAL);
-		slider.fill_size  = (slider.horizontal ? width: height) * (1 / ((adjustment->upper - adjustment->lower) / (adjustment->value - adjustment->lower)));	
-
+		slider.fill_size  = ((slider.horizontal ? width : height) - slider_length) * (1 / ((adjustment->upper - adjustment->lower) / (adjustment->value - adjustment->lower))) + slider_length / 2;
+		if (slider.horizontal)
+			slider.inverted = slider.inverted != (get_direction (widget) == GTK_TEXT_DIR_RTL);
 		clearlooks_draw_scale_trough (cr, &clearlooks_style->colors,
 		                              &params, &slider,
 		                              x, y, width, height);
@@ -570,37 +636,40 @@ draw_box (DRAW_ARGS)
 	{
 		WidgetParameters      params;
 		ProgressBarParameters progressbar;
-		gint8                 frame = 0;
+		gdouble               elapsed = 0.0;
 
 #ifdef HAVE_ANIMATION
 		if(clearlooks_style->animation && CL_IS_PROGRESS_BAR (widget))
 		{	
 			gboolean activity_mode = GTK_PROGRESS (widget)->activity_mode;
 			
-		 	if (!activity_mode &&
-			    gtk_progress_bar_get_fraction (GTK_PROGRESS_BAR (widget)) != 1.0 &&
-			    !cl_async_animation_lookup((gconstpointer)widget))
-			{
-				cl_progressbar_add ((gpointer)widget);
-			}
+		 	if (!activity_mode)
+				clearlooks_animation_progressbar_add ((gpointer)widget);
 		}
+
+		elapsed = clearlooks_animation_elapsed (widget);
 #endif
 		
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 
 		if (widget && GTK_IS_PROGRESS_BAR (widget))
+		{
 			progressbar.orientation = gtk_progress_bar_get_orientation (GTK_PROGRESS_BAR (widget));
+			progressbar.value = gtk_progress_bar_get_fraction(GTK_PROGRESS_BAR(widget));
+			progressbar.pulsing = GTK_PROGRESS (widget)->activity_mode;
+		}
 		else
+		{
 			progressbar.orientation = CL_ORIENTATION_LEFT_TO_RIGHT;
-		
-		cairo_reset_clip (cr);
-		
-#ifdef HAVE_ANIMATION
-		frame = 10 - cl_async_animation_getdata((gpointer)widget).frame;
-#endif
+			progressbar.value = 0;
+			progressbar.pulsing = FALSE;
+		}
+		/* WTF is this doing here?
+		 * cairo_reset_clip (cr); */
 		
 		clearlooks_draw_progressbar_fill (cr, colors, &params, &progressbar,
-		                                  x, y, width, height, frame);
+		                                  x, y, width, height,
+		                                  10 - (int)(elapsed * 10.0) % 10);
 	}
 	else if (DETAIL ("hscale") || DETAIL ("vscale"))
 	{
@@ -642,24 +711,25 @@ draw_box (DRAW_ARGS)
 		WidgetParameters params;
 		clearlooks_set_widget_parameters (widget, style, state_type, &params);
 		
-	/*	if (widget && GTK_IS_MENU_BAR (widget->parent))
+		if (widget && GTK_IS_MENU_BAR (widget->parent))
 		{
-			params.corners = CL_CORNER_NONE;
+			/*params.corners = CL_CORNER_NONE;
 			params.active = TRUE;
 			params.state_type = CL_STATE_ACTIVE;
 			params.xthickness = 2;
 			params.ythickness = 2;
 			
-			clearlooks_draw_button (cr, colors, &params, x, y, width, height+1);
+			clearlooks_draw_button (cr, colors, &params, x, y, width, height+1);*/
+			
+			params.corners = CL_CORNER_TOPLEFT | CL_CORNER_TOPRIGHT;
+			height += 1;
 		}
-		else*/
+		else
 		{	
-			WidgetParameters params;
-			clearlooks_set_widget_parameters (widget, style, state_type, &params);
-		
-			clearlooks_draw_menuitem (cr, colors, &params,
-			                          x, y, width, height);
+			params.corners = CL_CORNER_ALL;
 		}
+		
+		clearlooks_draw_menuitem (cr, colors, &params, x, y, width, height);
 	}
 	else if (DETAIL ("hscrollbar") || DETAIL ("vscrollbar") || DETAIL ("slider") || DETAIL ("stepper"))
 	{
@@ -701,7 +771,8 @@ draw_box (DRAW_ARGS)
 	}
 	else if (DETAIL ("toolbar") || DETAIL ("handlebox_bin") || DETAIL ("dockitem_bin"))
 	{
-		if (shadow_type != GTK_SHADOW_NONE)
+		// Only draw the shadows on horizontal toolbars
+		if (shadow_type != GTK_SHADOW_NONE && height < 2*width )
 			clearlooks_draw_toolbar (cr, colors, NULL, x, y, width, height);
 	}
 	else if (DETAIL ("trough"))
@@ -752,12 +823,8 @@ draw_option (DRAW_ARGS)
 	cairo_pattern_t *pt;
 
 #ifdef HAVE_ANIMATION
-	if(clearlooks_style->animation && GTK_IS_CHECK_BUTTON (widget) && !cl_async_animation_lookup((gconstpointer)widget)  && !g_slist_find (signaled_widgets, (gconstpointer)widget))
-	{
-			signaled_widgets = g_slist_append (signaled_widgets, widget);
-			g_signal_connect ((GObject*)widget, "toggled", G_CALLBACK (cl_checkbox_toggle), widget);
-			//printf("signal connected %d\n",widget);
-	}
+	if (clearlooks_style->animation)
+		clearlooks_animation_connect_checkbox (widget);
 #endif
 
 	if (state_type == GTK_STATE_INSENSITIVE)
@@ -799,19 +866,22 @@ draw_option (DRAW_ARGS)
 	cairo_stroke (cr);
 	
 #ifdef HAVE_ANIMATION
-	if (clearlooks_style->animation && cl_async_animation_lookup((gconstpointer)widget) && GTK_IS_CHECK_BUTTON (widget))
+	if (clearlooks_style->animation && GTK_IS_CHECK_BUTTON (widget) &&
+        clearlooks_animation_is_animated (widget) &&
+        !gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (widget)))
 	{
-		int value = cl_async_animation_getdata((gpointer)widget).frame;
-		
-		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-			trans = (float)(5-value)/5;
+		gfloat elapsed = clearlooks_animation_elapsed (widget);
+	
+		if (draw_bullet)
+			trans = sqrt (sqrt (MIN(elapsed / CHECK_ANIMATION_TIME, 1.0)));	
 		else
-			trans = (float)value/5;
+			trans = 1.0 - sqrt (sqrt (MIN(elapsed / CHECK_ANIMATION_TIME, 1.0)));
 		
 		draw_bullet = TRUE;
 	}
 #endif
 
+	/* inconsistent state is missing? */
 	if (draw_bullet)
 	{
 		cairo_arc (cr, 7, 7, 3, 0, M_PI*2);
@@ -835,17 +905,14 @@ draw_check (DRAW_ARGS)
 	CairoColor *border;
 	CairoColor *dot;
 	double trans = 1.0;
-	gboolean draw_bullet = (shadow_type == GTK_SHADOW_IN);
+	gboolean draw_bullet = (shadow_type == GTK_SHADOW_IN || shadow_type == GTK_SHADOW_ETCHED_IN);
 
 	cairo_t *cr = clearlooks_begin_paint (window, area);
 	cairo_pattern_t *pt;
 
 #ifdef HAVE_ANIMATION
-	if(clearlooks_style->animation && GTK_IS_CHECK_BUTTON (widget) && !cl_async_animation_lookup((gconstpointer)widget)  && !g_slist_find (signaled_widgets, (gconstpointer)widget))
-	{
-			signaled_widgets = g_slist_append (signaled_widgets, widget);
-			g_signal_connect ((GObject*)widget, "toggled", G_CALLBACK (cl_checkbox_toggle), widget);
-	}
+	if (clearlooks_style->animation)
+		clearlooks_animation_connect_checkbox (widget);
 #endif
 	
 	if (state_type == GTK_STATE_INSENSITIVE)
@@ -895,14 +962,16 @@ draw_check (DRAW_ARGS)
 	cairo_stroke (cr);
 		
 #ifdef HAVE_ANIMATION
-	if (clearlooks_style->animation && cl_async_animation_lookup((gconstpointer)widget))
+	if (clearlooks_style->animation && GTK_IS_CHECK_BUTTON (widget) &&
+	    clearlooks_animation_is_animated(widget) &&
+	    !gtk_toggle_button_get_inconsistent (GTK_TOGGLE_BUTTON (widget)))
 	{
-		int value = cl_async_animation_getdata((gpointer)widget).frame;
-		
-		if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)))
-			trans = (float)(5-value)/5;
+		gfloat elapsed = clearlooks_animation_elapsed (widget);
+	
+		if (draw_bullet)
+			trans = sqrt (sqrt (MIN(elapsed / CHECK_ANIMATION_TIME, 1.0)));
 		else
-			trans = (float)value/5;
+			trans = 1.0 - sqrt (sqrt (MIN(elapsed / CHECK_ANIMATION_TIME, 1.0)));
 		
 		draw_bullet = TRUE;
 	}
@@ -910,14 +979,22 @@ draw_check (DRAW_ARGS)
 
 	if (draw_bullet)
 	{
-		cairo_set_line_width (cr, 1.7);
-		cairo_move_to (cr, 0.5 + (width*0.2), (height*0.5));
-		cairo_line_to (cr, 0.5 + (width*0.4), (height*0.7));
-		
-		cairo_curve_to (cr, 0.5 + (width*0.4), (height*0.7),
-		                    0.5 + (width*0.5), (height*0.4),
-		                    0.5 + (width*0.70), (height*0.25));
+		if (shadow_type == GTK_SHADOW_ETCHED_IN) /* Inconsistent */
+		{
+			cairo_set_line_width (cr, 2.0);
+			cairo_move_to (cr, 3, height*0.5);
+			cairo_line_to (cr, width-3, height*0.5);
+		}
+		else
+		{
+			cairo_set_line_width (cr, 1.7);
+			cairo_move_to (cr, 0.5 + (width*0.2), (height*0.5));
+			cairo_line_to (cr, 0.5 + (width*0.4), (height*0.7));
 		
+			cairo_curve_to (cr, 0.5 + (width*0.4), (height*0.7),
+			                    0.5 + (width*0.5), (height*0.4),
+			                    0.5 + (width*0.70), (height*0.25));
+		}
 		
 		cairo_set_source_rgba (cr, dot->r, dot->g, dot->b, trans);
 		cairo_stroke (cr);
@@ -1052,6 +1129,31 @@ draw_resize_grip (GtkStyle       *style,
                   gint            width,
                   gint            height)
 {
+	ClearlooksStyle *clearlooks_style = CLEARLOOKS_STYLE (style);
+	ClearlooksColors *colors = &clearlooks_style->colors;
+
+	cairo_t *cr;
+
+	WidgetParameters params;
+	ResizeGripParameters grip;
+	grip.edge = (ClearlooksWindowEdge)edge;
+
+	int lx, ly;
+
+	g_return_if_fail (window != NULL);
+
+	if (edge != GDK_WINDOW_EDGE_SOUTH_EAST)
+		return; // sorry... need to work on this :P
+	
+	cr = clearlooks_begin_paint (window, area);
+
+	clearlooks_set_widget_parameters (widget, style, state_type, &params);	
+
+	clearlooks_draw_resize_grip (cr, colors, &params, &grip,
+	                             x, y, width, height);
+
+	cairo_destroy (cr);
+
 	return;
 }
 
@@ -1271,36 +1373,164 @@ draw_focus (GtkStyle *style, GdkWindow *window, GtkStateType state_type,
 		g_free (dash_list);
 }
 
+static void
+clearlooks_style_copy (GtkStyle * style, GtkStyle * src)
+{
+	ClearlooksStyle * cl_style = CLEARLOOKS_STYLE (style);
+	ClearlooksStyle * cl_src = CLEARLOOKS_STYLE (src);
+	
+	cl_style->colors              = cl_src->colors;
+	cl_style->progressbarstyle    = cl_src->progressbarstyle;
+	cl_style->menubarstyle        = cl_src->menubarstyle;
+	cl_style->menuitemstyle       = cl_src->menuitemstyle;
+	cl_style->listviewitemstyle   = cl_src->listviewitemstyle;
+	cl_style->scrollbar_color     = cl_src->scrollbar_color;
+	cl_style->has_scrollbar_color = cl_src->has_scrollbar_color;
+	cl_style->animation           = cl_src->animation;
+	
+	parent_class->copy (style, src);
+}
+
 static void
 clearlooks_style_unrealize (GtkStyle * style)
 {
-#ifdef HAVE_ANIMATION
-	 while (signaled_widgets)
-	{
-		if(GTK_IS_CHECK_BUTTON (signaled_widgets->data))
-			g_object_disconnect (signaled_widgets->data,
-			                     "any_signal::toggled",
-			                     G_CALLBACK (cl_checkbox_toggle),
-			                     signaled_widgets->data, NULL);
+	parent_class->unrealize (style);
+}
 
-		cl_async_animation_remove(signaled_widgets->data);
-		signaled_widgets = g_slist_next (signaled_widgets);
+static GdkPixbuf *
+set_transparency (const GdkPixbuf *pixbuf, gdouble alpha_percent)
+{
+	GdkPixbuf *target;
+	guchar *data, *current;
+	guint x, y, rowstride, height, width;
+
+	g_return_val_if_fail (pixbuf != NULL, NULL);
+	g_return_val_if_fail (GDK_IS_PIXBUF (pixbuf), NULL);
+
+	/* Returns a copy of pixbuf with it's non-completely-transparent pixels to
+	   have an alpha level "alpha_percent" of their original value. */
+
+	target = gdk_pixbuf_add_alpha (pixbuf, FALSE, 0, 0, 0);
+
+	if (alpha_percent == 1.0)
+		return target;
+	width = gdk_pixbuf_get_width (target);
+	height = gdk_pixbuf_get_height (target);
+	rowstride = gdk_pixbuf_get_rowstride (target);
+	data = gdk_pixbuf_get_pixels (target);
+
+	for (y = 0; y < height; y++) {
+		for (x = 0; x < width; x++) {
+			/* The "4" is the number of chars per pixel, in this case, RGBA,
+			   the 3 means "skip to the alpha" */
+			current = data + (y * rowstride) + (x * 4) + 3; 
+			*(current) = (guchar) (*(current) * alpha_percent);
+		}
 	}
 
-	if(async_widgets != NULL)
-	{
-		g_hash_table_foreach_remove(async_widgets, cl_disconnect, NULL);
-		//g_hash_table_destroy(async_widgets);
+	return target;
+}
+
+static GdkPixbuf*
+scale_or_ref (GdkPixbuf *src,
+              int width,
+              int height)
+{
+	if (width == gdk_pixbuf_get_width (src) &&
+	    height == gdk_pixbuf_get_height (src)) {
+		return g_object_ref (src);
+	} else {
+		return gdk_pixbuf_scale_simple (src,
+                                        width, height,
+                                        GDK_INTERP_BILINEAR);
+	}
+}
+
+static GdkPixbuf *
+render_icon (GtkStyle            *style,
+             const GtkIconSource *source,
+             GtkTextDirection     direction,
+             GtkStateType         state,
+             GtkIconSize          size,
+             GtkWidget           *widget,
+             const char          *detail)
+{
+	int width = 1;
+	int height = 1;
+	GdkPixbuf *scaled;
+	GdkPixbuf *stated;
+	GdkPixbuf *base_pixbuf;
+	GdkScreen *screen;
+	GtkSettings *settings;
+	
+	/* Oddly, style can be NULL in this function, because
+	 * GtkIconSet can be used without a style and if so
+	 * it uses this function.
+	 */
+	
+	base_pixbuf = gtk_icon_source_get_pixbuf (source);
+	
+	g_return_val_if_fail (base_pixbuf != NULL, NULL);
+	
+	if (widget && gtk_widget_has_screen (widget)) {
+		screen = gtk_widget_get_screen (widget);
+		settings = gtk_settings_get_for_screen (screen);
+	} else if (style->colormap) {
+		screen = gdk_colormap_get_screen (style->colormap);
+		settings = gtk_settings_get_for_screen (screen);
+	} else {
+		settings = gtk_settings_get_default ();
+		GTK_NOTE (MULTIHEAD,
+			  g_warning ("Using the default screen for gtk_default_render_icon()"));
 	}
 	
-	if(async_widget_timer_id != 0)
-	{
-		g_source_remove(async_widget_timer_id);
-		async_widget_timer_id = 0;
+  
+	if (size != (GtkIconSize) -1 && !gtk_icon_size_lookup_for_settings (settings, size, &width, &height)) {
+		g_warning (G_STRLOC ": invalid icon size '%d'", size);
+		return NULL;
 	}
-#endif /* HAVE_ANIMATION */	
 
-	parent_class->unrealize (style);
+	/* If the size was wildcarded, and we're allowed to scale, then scale; otherwise,
+	 * leave it alone.
+	 */
+	if (size != (GtkIconSize)-1 && gtk_icon_source_get_size_wildcarded (source))
+		scaled = scale_or_ref (base_pixbuf, width, height);
+	else
+		scaled = g_object_ref (base_pixbuf);
+	
+	/* If the state was wildcarded, then generate a state. */
+	if (gtk_icon_source_get_state_wildcarded (source)) {
+		if (state == GTK_STATE_INSENSITIVE) {
+			stated = set_transparency (scaled, 0.3);
+#if 0
+			stated =
+				gdk_pixbuf_composite_color_simple (scaled,
+								   gdk_pixbuf_get_width (scaled),
+								   gdk_pixbuf_get_height (scaled),
+								   GDK_INTERP_BILINEAR, 128,
+								   gdk_pixbuf_get_width (scaled),
+								   style->bg[state].pixel,
+								   style->bg[state].pixel);
+#endif
+			gdk_pixbuf_saturate_and_pixelate (stated, stated,
+							  0.1, FALSE);
+			
+			g_object_unref (scaled);
+		} else if (state == GTK_STATE_PRELIGHT) {
+			stated = gdk_pixbuf_copy (scaled);      
+			
+			gdk_pixbuf_saturate_and_pixelate (scaled, stated,
+							  1.2, FALSE);
+			
+			g_object_unref (scaled);
+		} else {
+			stated = scaled;
+		}
+	}
+	else
+		stated = scaled;
+  
+  return stated;
 }
 
 static void
@@ -1315,6 +1545,7 @@ clearlooks_style_class_init (ClearlooksStyleClass * klass)
 	
 	parent_class = g_type_class_peek_parent (klass);
 
+	style_class->copy             = clearlooks_style_copy;
 	style_class->unrealize        = clearlooks_style_unrealize;
 	style_class->init_from_rc     = clearlooks_style_init_from_rc;
 	style_class->draw_handle      = draw_handle;
@@ -1333,6 +1564,7 @@ clearlooks_style_class_init (ClearlooksStyleClass * klass)
 	style_class->draw_hline       = draw_hline;
 	style_class->draw_resize_grip = draw_resize_grip;
 	style_class->draw_arrow       = draw_arrow;
+	style_class->render_icon      = render_icon;
 }
 
 GType clearlooks_type_style = 0;
