@@ -61,13 +61,14 @@ redmond_draw_hline (GtkStyle * style,
             gint x2, 
             gint y)
 {
-  g_return_if_fail(sanitize_parameters(style, window, NULL, NULL));
+  RedmondStyle *redmond_style = REDMOND_STYLE (style);
+  CHECK_ARGS
  
-  cairo_t *cr = redmond_begin_paint (window, area);
+  cairo_t *cr = ge_gdk_drawable_to_cairo (window, area);
 
   do_redmond_draw_line(cr, 
-                       &style->dark[state_type], 
-                       &style->light[state_type], 
+                       &redmond_style->color_cube.dark[state_type], 
+                       &redmond_style->color_cube.light[state_type], 
                        area, 
                        x1, x2, y, 
                        TRUE);
@@ -91,16 +92,17 @@ redmond_draw_vline (GtkStyle * style,
             gint y2, 
             gint x)
 {
-  if (is_combo_box(widget, FALSE) && (!is_combo_box_entry(widget)))
+  RedmondStyle *redmond_style = REDMOND_STYLE (style);
+  if (ge_is_combo_box(widget, FALSE) && (!ge_is_combo_box_entry(widget)))
     return;
  
-  g_return_if_fail(sanitize_parameters(style, window, NULL, NULL));
+  CHECK_ARGS
  
-  cairo_t *cr = redmond_begin_paint (window, area);
+  cairo_t *cr = ge_gdk_drawable_to_cairo (window, area);
 
   do_redmond_draw_line(cr, 
-                       &style->dark[state_type], 
-                       &style->light[state_type], 
+                       &redmond_style->color_cube.dark[state_type], 
+                       &redmond_style->color_cube.light[state_type], 
                        area, 
                        y1, y2, x, 
                        FALSE);
@@ -139,7 +141,9 @@ redmond_draw_check (GtkStyle * style,
             gint width, 
             gint height)
 {
-  g_return_if_fail(sanitize_parameters(style, window, &width, &height));
+  RedmondStyle *redmond_style = REDMOND_STYLE (style);
+  CHECK_ARGS
+  SANITIZE_SIZE
  
   /* Always draw the checkbox and mark centered
    * at a fixed size.
@@ -149,7 +153,7 @@ redmond_draw_check (GtkStyle * style,
   width = CHECK_SIZE;
   height = CHECK_SIZE;*/
   
-  cairo_t *cr = redmond_begin_paint (window, area);
+  cairo_t *cr = ge_gdk_drawable_to_cairo (window, area);
 
   if (CHECK_DETAIL (detail, "check"))	/* Menu item */
     {
@@ -158,7 +162,7 @@ redmond_draw_check (GtkStyle * style,
        */
       if (shadow == GTK_SHADOW_IN)
 	{
-	  do_redmond_draw_check (cr, &style->text[state], x + 3, y + 3,
+	  do_redmond_draw_check (cr, &redmond_style->color_cube.text[state], x + 3, y + 3,
 			 width - 4, height - 4);
 	}
     }
@@ -172,12 +176,22 @@ redmond_draw_check (GtkStyle * style,
 	    do_redmond_draw_cross_hatch_fill (style, window, GTK_STATE_NORMAL, 
                                               area, RADIO_NONE, x, y, width, height);
 	  else
-	    gdk_draw_rectangle (window, style->base_gc[GTK_STATE_NORMAL],
-			        TRUE, x, y, width, height);
+          {
+            ge_cairo_set_color(cr, &redmond_style->color_cube.base[GTK_STATE_NORMAL]);
+
+            cairo_rectangle(cr, x, y, width - 1, height - 1);
+
+            cairo_fill(cr);
+          }
+
 	  break;
 	default:
-	  gdk_draw_rectangle (window, style->bg_gc[GTK_STATE_NORMAL], TRUE,
-			      x, y, width, height);
+          ge_cairo_set_color(cr, &redmond_style->color_cube.bg[GTK_STATE_NORMAL]);
+
+          cairo_rectangle(cr, x, y, width - 1, height - 1);
+
+          cairo_fill(cr);
+
 	}
  
       if ((shadow == GTK_SHADOW_IN) || 
@@ -188,17 +202,17 @@ redmond_draw_check (GtkStyle * style,
               (IS_TOGGLE_BUTTON(widget) && 
                TOGGLE_BUTTON(widget)->inconsistent))
             {
-	      do_redmond_draw_check (cr, &style->fg[GTK_STATE_INSENSITIVE],
+	      do_redmond_draw_check (cr, &redmond_style->color_cube.fg[GTK_STATE_INSENSITIVE],
 			             x + 2, y + 2, width - 4, height - 4);
             }
 	  else
             {
-	      do_redmond_draw_check (cr, &style->fg[GTK_STATE_NORMAL],
+	      do_redmond_draw_check (cr, &redmond_style->color_cube.fg[GTK_STATE_NORMAL],
 			             x + 2, y + 2, width - 4, height - 4);
             }
 	}
  
-      gtk_paint_shadow (style, window, GTK_STATE_NORMAL, GTK_SHADOW_IN,
+      redmond_draw_shadow (style, window, GTK_STATE_NORMAL, GTK_SHADOW_IN,
 			area, widget, detail, x, y, width, height);
     }
     
@@ -238,7 +252,8 @@ redmond_draw_option (GtkStyle * style,
 {
   RedmondStyle *redmond_style = REDMOND_STYLE (style);
  
-  g_return_if_fail(sanitize_parameters(style, window, &width, &height));
+  CHECK_ARGS
+  SANITIZE_SIZE
  
   /* Since X arcs are very primitive and don't always
    * produce circles, we use bits here. Since the bits
@@ -330,20 +345,23 @@ redmond_draw_arrow (GtkStyle * style,
             gint width, 
             gint height)
 {
+  RedmondStyle *redmond_style = REDMOND_STYLE (style);
+
   gboolean button_in = (shadow == GTK_SHADOW_IN);
   gint child_offset_x = 1, child_offset_y = 1;
  
-  if (is_combo_box(widget, FALSE) && (!is_combo_box_entry(widget)))
+  if (ge_is_combo_box(widget, FALSE) && (!ge_is_combo_box_entry(widget)))
     return;
  
-  g_return_if_fail(sanitize_parameters(style, window, &width, &height));
+  CHECK_ARGS
+  SANITIZE_SIZE
 
   if ((CHECK_DETAIL (detail, "spinbutton"))
       || (CHECK_DETAIL (detail, "vscrollbar"))
       || (CHECK_DETAIL (detail, "hscrollbar"))
       || (CHECK_DETAIL (detail, "optionmenu"))
       || IS_SCROLLBAR(widget)
-      || (is_in_combo_box (widget)))
+      || (ge_is_in_combo_box (widget)))
     {
       if (state != GTK_STATE_INSENSITIVE)
         state = GTK_STATE_NORMAL;
@@ -353,7 +371,7 @@ redmond_draw_arrow (GtkStyle * style,
 	  if ((!widget) || (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR))
 	    x -= 1;
 	}
-      else if (is_in_combo_box (widget) && ((gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)))
+      else if (ge_is_in_combo_box (widget) && ((gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL)))
 	{
 	    x += 1;
 	}
@@ -362,7 +380,7 @@ redmond_draw_arrow (GtkStyle * style,
        * for ComboBox/ComboBoxEntry, because its cleaner,
        * even though it seems slightly inconsistant.
        */
-      if (widget && is_in_combo_box (widget))
+      if (widget && ge_is_in_combo_box (widget))
 	{
 	  y += 1;
 	  width -= 2;
@@ -383,15 +401,15 @@ redmond_draw_arrow (GtkStyle * style,
         state = GTK_STATE_NORMAL;
     }
  
-  cairo_t *cr = redmond_begin_paint (window, area);
+  cairo_t *cr = ge_gdk_drawable_to_cairo (window, area);
 
   if (state == GTK_STATE_INSENSITIVE)
     {
-      do_redmond_draw_arrow (cr, &style->light[state], arrow_type, x+1, y+1, width, height);
-      do_redmond_draw_arrow (cr, &style->fg[state], arrow_type, x, y, width, height);
+      do_redmond_draw_arrow (cr, &redmond_style->color_cube.light[state], arrow_type, x+1, y+1, width, height);
+      do_redmond_draw_arrow (cr, &redmond_style->color_cube.fg[state], arrow_type, x, y, width, height);
     }
   else
-    do_redmond_draw_arrow (cr, &style->fg[state], arrow_type, x, y, width, height);
+    do_redmond_draw_arrow (cr, &redmond_style->color_cube.fg[state], arrow_type, x, y, width, height);
 
   cairo_destroy(cr);
 }
@@ -421,29 +439,30 @@ redmond_draw_shadow (GtkStyle * style,
 {
   RedmondStyle *redmond_style = REDMOND_STYLE (style);
  
-  g_return_if_fail(sanitize_parameters(style, window, &width, &height));
+  CHECK_ARGS
+  SANITIZE_SIZE
  
   if (shadow_type == GTK_SHADOW_NONE)
     return;
 
-  cairo_t *cr = redmond_begin_paint (window, area);
+  cairo_t *cr = ge_gdk_drawable_to_cairo (window, area);
   
   switch (shadow_type)
     {
     case GTK_SHADOW_NONE:
       break;
     case GTK_SHADOW_ETCHED_IN:
-      do_redmond_draw_shadow (cr, &style->dark[state_type],
-		      &style->light[state_type], x, y, width, height, TRUE);
-      do_redmond_draw_shadow (cr, &style->light[state_type],
-		      &style->dark[state_type], x + 1, y + 1, width - 2,
+      ge_cairo_simple_border (cr, &redmond_style->color_cube.dark[state_type],
+		      &redmond_style->color_cube.light[state_type], x, y, width, height, TRUE);
+      ge_cairo_simple_border (cr, &redmond_style->color_cube.light[state_type],
+		      &redmond_style->color_cube.dark[state_type], x + 1, y + 1, width - 2,
 		      height - 2, TRUE);
       break;
     case GTK_SHADOW_ETCHED_OUT:
-      do_redmond_draw_shadow (cr, &style->light[state_type],
-		      &style->dark[state_type], x, y, width, height, TRUE);
-      do_redmond_draw_shadow (cr, &style->dark[state_type],
-		      &style->light[state_type], x + 1, y + 1, width - 2,
+      ge_cairo_simple_border (cr, &redmond_style->color_cube.light[state_type],
+		      &redmond_style->color_cube.dark[state_type], x, y, width, height, TRUE);
+      ge_cairo_simple_border (cr, &redmond_style->color_cube.dark[state_type],
+		      &redmond_style->color_cube.light[state_type], x + 1, y + 1, width - 2,
 		      height - 2, TRUE);
       break;
     case GTK_SHADOW_IN:
@@ -455,7 +474,7 @@ redmond_draw_shadow (GtkStyle * style,
 	   * it only draws exactly one pixel around button.
 	   */
            cairo_set_line_width (cr, 1);
-           gdk_cairo_set_source_color(cr, &redmond_style->black_border[state_type]);	
+           ge_cairo_set_color(cr, &redmond_style->black_border[state_type]);	
            cairo_rectangle(cr, x + 0.5, y + 0.5, width - 1, height - 1);
            cairo_stroke(cr);
 	}
@@ -468,14 +487,14 @@ redmond_draw_shadow (GtkStyle * style,
 	   * dark edge on pressed.
 	   */
            cairo_set_line_width (cr, 1);
-           gdk_cairo_set_source_color(cr, &style->dark[state_type]);	
+           ge_cairo_set_color(cr, &redmond_style->color_cube.dark[state_type]);	
            cairo_rectangle(cr, x + 0.5, y + 0.5, width - 1, height - 1);
            cairo_stroke(cr);
 	}
       else if (((CHECK_DETAIL (detail, "entry"))
 		|| (CHECK_DETAIL (detail, "frame")))
 	       && widget && ((IS_SPIN_BUTTON (widget))
-			     || (is_in_combo_box (widget))))
+			     || (ge_is_in_combo_box (widget))))
 	{
 	  /* The Combo/ComboBoxEntry button and the SpingButton Steppers should apear
 	   * to be inset into the entry, as opposed to next to it, so we fake it
@@ -485,22 +504,22 @@ redmond_draw_shadow (GtkStyle * style,
  
 	  if ((!widget) || (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR))
 	    {
-	      do_redmond_draw_shadow (cr, &style->dark[state_type],
-			      &style->light[state_type], x, y, width + 2,
+	      ge_cairo_simple_border (cr, &redmond_style->color_cube.dark[state_type],
+			      &redmond_style->color_cube.light[state_type], x, y, width + 2,
 			      height, FALSE);
-	      do_redmond_draw_shadow (cr,
+	      ge_cairo_simple_border (cr,
 			      &redmond_style->black_border[state_type],
-			      &style->bg[state_type], x + 1, y + 1, width,
+			      &redmond_style->color_cube.bg[state_type], x + 1, y + 1, width,
 			      height - 2, FALSE);
 	    }
 	  else
 	    {
-	      do_redmond_draw_shadow (cr, &style->dark[state_type],
-			      &style->light[state_type], x - 2, y,
+	      ge_cairo_simple_border (cr, &redmond_style->color_cube.dark[state_type],
+			      &redmond_style->color_cube.light[state_type], x - 2, y,
 			      width + 2, height, FALSE);
-	      do_redmond_draw_shadow (cr,
+	      ge_cairo_simple_border (cr,
 			      &redmond_style->black_border[state_type],
-			      &style->bg[state_type], x - 1, y + 1, width,
+			      &redmond_style->color_cube.bg[state_type], x - 1, y + 1, width,
 			      height - 2, FALSE);
 	    }
 	}
@@ -508,22 +527,22 @@ redmond_draw_shadow (GtkStyle * style,
 	       || ((IS_STATUS_BAR (widget)))
 	       || ((CHECK_DETAIL (detail, "frame")))
 	       || ((CHECK_DETAIL (detail, "button"))
-		   && (is_toolbar_item (widget))))
+		   && (ge_is_toolbar_item (widget))))
 	{
 	  /* Toolbar Buttons, Status Bars, Frames, and Troughs are drawn with a thin border */
-	  do_redmond_draw_shadow (cr, &style->dark[state_type],
-			  &style->light[state_type], x, y, width, height,
+	  ge_cairo_simple_border (cr, &redmond_style->color_cube.dark[state_type],
+			  &redmond_style->color_cube.light[state_type], x, y, width, height,
 			  FALSE);
 	}
       else
 	{
 	  /* Normal IN Border */
-	  do_redmond_draw_shadow (cr, &style->dark[state_type],
-			  &style->light[state_type], x, y, width, height,
+	  ge_cairo_simple_border (cr, &redmond_style->color_cube.dark[state_type],
+			  &redmond_style->color_cube.light[state_type], x, y, width, height,
 			  FALSE);
-	  do_redmond_draw_shadow (cr,
+	  ge_cairo_simple_border (cr,
 			  &redmond_style->black_border[state_type],
-			  &style->bg[state_type], x + 1, y + 1, width - 2,
+			  &redmond_style->color_cube.bg[state_type], x + 1, y + 1, width - 2,
 			  height - 2, FALSE);
 	}
       break;
@@ -533,36 +552,36 @@ redmond_draw_shadow (GtkStyle * style,
 	  || (CHECK_DETAIL (detail, "slider"))
 	  || (CHECK_DETAIL (detail, "spinbutton_up"))
 	  || (CHECK_DETAIL (detail, "spinbutton_down"))
-	  || ((CHECK_DETAIL (detail, "button")) && is_in_combo_box (widget)))
+	  || ((CHECK_DETAIL (detail, "button")) && ge_is_in_combo_box (widget)))
 	{
 	  /* In the case of Scrollbars steppers, Sliders, Spinbutton steppers,
 	   * and Combo/ComboBoxEntry buttons, the internal & external borders are 
 	   * inverted when OUT since they are inset into another widgets IN edge.
 	   */
-	  do_redmond_draw_shadow (cr, &style->bg[state_type],
+	  ge_cairo_simple_border (cr, &redmond_style->color_cube.bg[state_type],
 			  &redmond_style->black_border[state_type], x, y,
 			  width, height, FALSE);
-	  do_redmond_draw_shadow (cr, &style->light[state_type],
-			  &style->dark[state_type], x + 1, y + 1, width - 2,
+	  ge_cairo_simple_border (cr, &redmond_style->color_cube.light[state_type],
+			  &redmond_style->color_cube.dark[state_type], x + 1, y + 1, width - 2,
 			  height - 2, FALSE);
 	}
       else if ((CHECK_DETAIL (detail, "frame"))
 	       || ((CHECK_DETAIL (detail, "button"))
-		   && (is_toolbar_item (widget))))
+		   && (ge_is_toolbar_item (widget))))
 	{
 	  /* Toolbar Buttons and Frames are drawn with a thin border */
-	  do_redmond_draw_shadow (cr, &style->light[state_type],
-			  &style->dark[state_type], x, y, width, height,
+	  ge_cairo_simple_border (cr, &redmond_style->color_cube.light[state_type],
+			  &redmond_style->color_cube.dark[state_type], x, y, width, height,
 			  FALSE);
 	}
       else
 	{
 	  /* Normal OUT Border */
-	  do_redmond_draw_shadow (cr, &style->light[state_type],
+	  ge_cairo_simple_border (cr, &redmond_style->color_cube.light[state_type],
 			  &redmond_style->black_border[state_type], x, y,
 			  width, height, FALSE);
-	  do_redmond_draw_shadow (cr, &style->bg[state_type],
-			  &style->dark[state_type], x + 1, y + 1, width - 2,
+	  ge_cairo_simple_border (cr, &redmond_style->color_cube.bg[state_type],
+			  &redmond_style->color_cube.dark[state_type], x + 1, y + 1, width - 2,
 			  height - 2, FALSE);
 	}
       break;
@@ -603,10 +622,10 @@ redmond_draw_combobox_button (GtkStyle * style,
 
   if ((!(widget)) || (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR))
     {
-      if (is_combo_box_entry (widget)
-          || is_combo_box (widget, TRUE))
+      if (ge_is_combo_box_entry (widget)
+          || ge_is_combo_box (widget, TRUE))
         {
-          if (!is_combo_box_entry (widget))
+          if (!ge_is_combo_box_entry (widget))
             {
               if ((widget->parent))
                 {
@@ -632,7 +651,7 @@ redmond_draw_combobox_button (GtkStyle * style,
           gtk_paint_shadow (style, window, state_type, GTK_SHADOW_IN,
   		            area, widget, "entry", x - 2, y, width, height);
         }
-      else if (is_combo(widget))
+      else if (ge_is_combo(widget))
         {
           GtkWidget *entry = widget;
 
@@ -714,10 +733,10 @@ redmond_draw_combobox_button (GtkStyle * style,
     } 
   else
     {
-      if (is_combo_box_entry (widget)
-          || is_combo_box (widget, TRUE))
+      if (ge_is_combo_box_entry (widget)
+          || ge_is_combo_box (widget, TRUE))
         {
-          if (!is_combo_box_entry (widget))
+          if (!ge_is_combo_box_entry (widget))
             {
               if ((widget->parent))
                 {
@@ -743,7 +762,7 @@ redmond_draw_combobox_button (GtkStyle * style,
           gtk_paint_shadow (style, window, state_type, GTK_SHADOW_IN,
 	  		    area, widget, "entry", x + 2, y, width, height);
         }
-      else if (is_combo(widget))
+      else if (ge_is_combo(widget))
         {
           GtkWidget *entry = widget;
 
@@ -946,7 +965,8 @@ redmond_draw_box (GtkStyle * style,
 {
   RedmondStyle *redmond_style = REDMOND_STYLE (style);
  
-  g_return_if_fail(sanitize_parameters(style, window, &width, &height));
+  CHECK_ARGS
+  SANITIZE_SIZE
  
   if (IS_MENU_SHELL(widget))
     {
@@ -1018,7 +1038,7 @@ redmond_draw_box (GtkStyle * style,
  
       if (((CHECK_DETAIL (detail, "dockitem_bin")) && 
            (IS_BONOBO_DOCK_ITEM(widget))) || 
-          ((widget) && (is_bonobo_dock_item(widget->parent))))
+          ((widget) && (ge_is_bonobo_dock_item(widget->parent))))
 	{	  
 	  GList *children = NULL, *child = NULL;
 	  GtkWidget *dockitem = widget;
@@ -1172,7 +1192,7 @@ redmond_draw_box (GtkStyle * style,
        */
       if ((widget) && (widget->parent) &&
 	  ((!((CHECK_DETAIL (detail, "menubar")) && 
-	  is_panel_widget_item (widget)))))
+	  ge_is_panel_widget_item (widget)))))
         {
           GdkRectangle shadow, clip;
           
@@ -1186,10 +1206,10 @@ redmond_draw_box (GtkStyle * style,
           else
             clip = shadow;
             
-           cairo_t *cr = redmond_begin_paint (window, &clip);
+           cairo_t *cr = ge_gdk_drawable_to_cairo (window, &clip);
           
-	  do_redmond_draw_shadow (cr, &style->light[state_type],
-			          &style->dark[state_type], x, y, 
+	  ge_cairo_simple_border (cr, &redmond_style->color_cube.light[state_type],
+			          &redmond_style->color_cube.dark[state_type], x, y, 
                                   width + 2*right_cutoff, 
                                   height + 2*bottom_cutoff,
 			          FALSE);
@@ -1204,8 +1224,7 @@ redmond_draw_box (GtkStyle * style,
        * a thin inset border on select/active,
        * and a thin outset border on prelight
        */
-      GdkColor *top, *bottom;
-      gboolean pressed = FALSE;
+      CairoColor *top, *bottom;
  
       if (state_type != GTK_STATE_INSENSITIVE)
 	state_type = GTK_STATE_NORMAL;
@@ -1220,18 +1239,18 @@ redmond_draw_box (GtkStyle * style,
              GTK_WIDGET_REALIZED(GTK_MENU(GTK_MENU_ITEM(widget)->submenu)->toplevel) &&
              GTK_WIDGET_VISIBLE(GTK_MENU(GTK_MENU_ITEM(widget)->submenu)->toplevel))))
         {  
-          top = &style->light[state_type];
-          bottom = &style->dark[state_type];
+          top = &redmond_style->color_cube.light[state_type];
+          bottom = &redmond_style->color_cube.dark[state_type];
         }
       else
         {
-          top = &style->dark[state_type];
-          bottom = &style->light[state_type];
+          top = &redmond_style->color_cube.dark[state_type];
+          bottom = &redmond_style->color_cube.light[state_type];
         }
 
-      cairo_t *cr = redmond_begin_paint (window, area);
+      cairo_t *cr = ge_gdk_drawable_to_cairo (window, area);
           
-      do_redmond_draw_shadow (cr, top, bottom,
+      ge_cairo_simple_border (cr, top, bottom,
 		      x, y, width, height,
 		      FALSE);
 
@@ -1252,7 +1271,7 @@ redmond_draw_box (GtkStyle * style,
                                     width, height);
     }
   else if ((CHECK_DETAIL (detail, "button")) && widget
-	   && is_in_combo_box (widget))
+	   && ge_is_in_combo_box (widget))
     {
       /* ComboBox Buttons are an especially complex case
        * so call our special function for them.
@@ -1305,7 +1324,7 @@ redmond_draw_box (GtkStyle * style,
     }
  
   if (CHECK_DETAIL(detail, "optionmenu") ||  (CHECK_DETAIL(detail, "button") && 
-       (is_combo_box(widget, FALSE)) && !(is_combo_box_entry(widget))))
+       (ge_is_combo_box(widget, FALSE)) && !(ge_is_combo_box_entry(widget))))
     {
       GtkRequisition indicator_size;
       GtkBorder indicator_spacing;
@@ -1314,7 +1333,7 @@ redmond_draw_box (GtkStyle * style,
       if (state_type != GTK_STATE_INSENSITIVE)
         state_type = GTK_STATE_NORMAL;
  
-      option_menu_get_props (widget, &indicator_size, &indicator_spacing);
+      ge_option_menu_get_props (widget, &indicator_size, &indicator_spacing);
  
       if ((!widget) || (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL))
 	vline_x = x + indicator_size.width + indicator_spacing.left + indicator_spacing.right;
@@ -1322,9 +1341,9 @@ redmond_draw_box (GtkStyle * style,
 	vline_x = x + width - (indicator_size.width + indicator_spacing.left + 
                                indicator_spacing.right) - style->xthickness;
 
-      cairo_t *cr = redmond_begin_paint (window, area);
+      cairo_t *cr = ge_gdk_drawable_to_cairo (window, area);
 
-      do_redmond_draw_line(cr, &style->dark[state_type], &style->light[state_type], 
+      do_redmond_draw_line(cr, &redmond_style->color_cube.dark[state_type], &redmond_style->color_cube.light[state_type], 
                            area, y + style->ythickness + 1, y + height - style->ythickness - 2, 
                            vline_x, FALSE);
  
@@ -1385,7 +1404,8 @@ redmond_draw_slider (GtkStyle * style,
              gint height, 
              GtkOrientation orientation)
 {
-  g_return_if_fail(sanitize_parameters(style, window, &width, &height));
+  CHECK_ARGS
+  SANITIZE_SIZE
  
   if ((CHECK_DETAIL (detail, "hscale")) || (CHECK_DETAIL (detail, "vscale")))
     {
@@ -1427,54 +1447,49 @@ redmond_draw_extension (GtkStyle * style,
 {
   RedmondStyle *redmond_style = REDMOND_STYLE (style);
  
-  GdkGC *gc1 = NULL;
-  GdkGC *gc2 = NULL;
-  GdkGC *gc3 = NULL;
-  GdkGC *gc4 = NULL;
+  CairoColor *color1 = NULL;
+  CairoColor *color2 = NULL;
+  CairoColor *color3 = NULL;
+  CairoColor *color4 = NULL;
  
-  g_return_if_fail(sanitize_parameters(style, window, &width, &height));
+  CHECK_ARGS
+  SANITIZE_SIZE
  
   do_redmond_draw_default_fill (style, window, GTK_STATE_NORMAL, area, 
                                 x, y, width, height);
  
+      cairo_t *cr = ge_gdk_drawable_to_cairo (window, area);
+
   switch (shadow_type)
     {
     case GTK_SHADOW_NONE:
       return;
     case GTK_SHADOW_IN:
-      gc1 = style->dark_gc[state_type];
-      gc2 = redmond_style->black_border_gc[state_type];
-      gc3 = style->bg_gc[state_type];
-      gc4 = style->light_gc[state_type];
+      color1 = &redmond_style->color_cube.dark[state_type];
+      color2 = &redmond_style->black_border[state_type];
+      color3 = &redmond_style->color_cube.bg[state_type];
+      color4 = &redmond_style->color_cube.light[state_type];
       break;
     case GTK_SHADOW_ETCHED_IN:
-      gc1 = style->dark_gc[state_type];
-      gc2 = style->light_gc[state_type];
-      gc3 = style->dark_gc[state_type];
-      gc4 = style->light_gc[state_type];
+      color1 = &redmond_style->color_cube.dark[state_type];
+      color2 = &redmond_style->color_cube.light[state_type];
+      color3 = &redmond_style->color_cube.dark[state_type];
+      color4 = &redmond_style->color_cube.light[state_type];
       break;
     case GTK_SHADOW_OUT:
-      gc1 = style->light_gc[state_type];
-      gc2 = style->bg_gc[state_type];
-      gc3 = style->dark_gc[state_type];
-      gc4 = redmond_style->black_border_gc[state_type];
+      color1 = &redmond_style->color_cube.light[state_type];
+      color2 = &redmond_style->color_cube.bg[state_type];
+      color3 = &redmond_style->color_cube.dark[state_type];
+      color4 = &redmond_style->black_border[state_type];
       break;
     case GTK_SHADOW_ETCHED_OUT:
-      gc1 = style->light_gc[state_type];
-      gc2 = style->dark_gc[state_type];
-      gc3 = style->light_gc[state_type];
-      gc4 = style->dark_gc[state_type];
+      color1 = &redmond_style->color_cube.light[state_type];
+      color2 = &redmond_style->color_cube.dark[state_type];
+      color3 = &redmond_style->color_cube.light[state_type];
+      color4 = &redmond_style->color_cube.dark[state_type];
       break;
     }
  
-  if (area)
-    {
-      gdk_gc_set_clip_rectangle (gc1, area);
-      gdk_gc_set_clip_rectangle (gc2, area);
-      gdk_gc_set_clip_rectangle (gc3, area);
-      gdk_gc_set_clip_rectangle (gc4, area);
-    }
- 
   switch (shadow_type)
     {
     case GTK_SHADOW_NONE:
@@ -1491,17 +1506,17 @@ redmond_draw_extension (GtkStyle * style,
                                         width - (2 * style->xthickness),
                                         height - (style->ythickness));
  
-	  gdk_draw_line (window, gc1, x, y, x, y + height - 2);
-	  gdk_draw_line (window, gc2, x + 1, y, x + 1, y + height - 2);
+	  ge_cairo_line (cr, color1, x, y, x, y + height - 2);
+	  ge_cairo_line (cr, color2, x + 1, y, x + 1, y + height - 2);
  
-	  gdk_draw_line (window, gc3,
+	  ge_cairo_line (cr, color3,
 			 x + 2, y + height - 2, x + width - 2,
 			 y + height - 2);
-	  gdk_draw_line (window, gc3, x + width - 2, y, x + width - 2,
+	  ge_cairo_line (cr, color3, x + width - 2, y, x + width - 2,
 			 y + height - 2);
-	  gdk_draw_line (window, gc4, x + 1, y + height - 1, x + width - 2,
+	  ge_cairo_line (cr, color4, x + 1, y + height - 1, x + width - 2,
 			 y + height - 1);
-	  gdk_draw_line (window, gc4, x + width - 1, y, x + width - 1,
+	  ge_cairo_line (cr, color4, x + width - 1, y, x + width - 1,
 			 y + height - 2);
 	  break;
 	case GTK_POS_BOTTOM:
@@ -1511,18 +1526,18 @@ redmond_draw_extension (GtkStyle * style,
                                         width - (2 * style->xthickness),
                                         height - (style->ythickness));
  
-	  gdk_draw_line (window, gc1, x + 2, y, x + width - 3, y);
-	  gdk_draw_line (window, gc1, x, y + 2, x, y + height - 1);
+	  ge_cairo_line (cr, color1, x + 2, y, x + width - 3, y);
+	  ge_cairo_line (cr, color1, x, y + 2, x, y + height - 1);
  
-	  gdk_draw_line (window, gc2, x + 1, y + 1, x + width - 2, y + 1);
-	  gdk_draw_line (window, gc2, x + 1, y + 1, x + 1, y + height - 1);
-	  gdk_draw_line (window, gc1, x, y + 2, x + 1, y + 1);
+	  ge_cairo_line (cr, color2, x + 1, y + 1, x + width - 2, y + 1);
+	  ge_cairo_line (cr, color2, x + 1, y + 1, x + 1, y + height - 1);
+	  ge_cairo_line (cr, color1, x, y + 2, x + 1, y + 1);
  
-	  gdk_draw_line (window, gc3,
+	  ge_cairo_line (cr, color3,
 			 x + width - 2, y + 2, x + width - 2, y + height - 1);
-	  gdk_draw_line (window, gc4, x + width - 1, y + 2, x + width - 1,
+	  ge_cairo_line (cr, color4, x + width - 1, y + 2, x + width - 1,
 			 y + height - 1);
-	  gdk_draw_line (window, gc4, x + width - 1, y + 2, x + width - 2,
+	  ge_cairo_line (cr, color4, x + width - 1, y + 2, x + width - 2,
 			 y + 1);
 	  break;
 	case GTK_POS_LEFT:
@@ -1532,16 +1547,16 @@ redmond_draw_extension (GtkStyle * style,
                                         width - (style->xthickness),
                                         height - (2 * style->ythickness));
  
-	  gdk_draw_line (window, gc1, x, y, x + width - 2, y);
-	  gdk_draw_line (window, gc2, x + 1, y + 1, x + width - 2, y + 1);
+	  ge_cairo_line (cr, color1, x, y, x + width - 2, y);
+	  ge_cairo_line (cr, color2, x + 1, y + 1, x + width - 2, y + 1);
  
-	  gdk_draw_line (window, gc3,
+	  ge_cairo_line (cr, color3,
 			 x, y + height - 2, x + width - 2, y + height - 2);
-	  gdk_draw_line (window, gc3,
+	  ge_cairo_line (cr, color3,
 			 x + width - 2, y + 2, x + width - 2, y + height - 2);
-	  gdk_draw_line (window, gc4, x, y + height - 1, x + width - 2,
+	  ge_cairo_line (cr, color4, x, y + height - 1, x + width - 2,
 			 y + height - 1);
-	  gdk_draw_line (window, gc4, x + width - 1, y + 1, x + width - 1,
+	  ge_cairo_line (cr, color4, x + width - 1, y + 1, x + width - 1,
 			 y + height - 2);
 	  break;
 	case GTK_POS_RIGHT:
@@ -1551,27 +1566,21 @@ redmond_draw_extension (GtkStyle * style,
                                         width - (style->xthickness),
                                         height - (2 * style->ythickness));
  
-	  gdk_draw_line (window, gc1, x + 1, y, x + width - 1, y);
-	  gdk_draw_line (window, gc1, x, y + 1, x, y + height - 2);
-	  gdk_draw_line (window, gc2, x + 1, y + 1, x + width - 1, y + 1);
-	  gdk_draw_line (window, gc2, x + 1, y + 1, x + 1, y + height - 2);
+	  ge_cairo_line (cr, color1, x + 1, y, x + width - 1, y);
+	  ge_cairo_line (cr, color1, x, y + 1, x, y + height - 2);
+	  ge_cairo_line (cr, color2, x + 1, y + 1, x + width - 1, y + 1);
+	  ge_cairo_line (cr, color2, x + 1, y + 1, x + 1, y + height - 2);
  
-	  gdk_draw_line (window, gc3,
+	  ge_cairo_line (cr, color3,
 			 x + 2, y + height - 2, x + width - 1,
 			 y + height - 2);
-	  gdk_draw_line (window, gc4, x + 1, y + height - 1, x + width - 1,
+	  ge_cairo_line (cr, color4, x + 1, y + height - 1, x + width - 1,
 			 y + height - 1);
 	  break;
 	}
     }
  
-  if (area)
-    {
-      gdk_gc_set_clip_rectangle (gc1, NULL);
-      gdk_gc_set_clip_rectangle (gc2, NULL);
-      gdk_gc_set_clip_rectangle (gc3, NULL);
-      gdk_gc_set_clip_rectangle (gc4, NULL);
-    }
+    cairo_destroy(cr);
 }
  
 /***********************************************
@@ -1594,9 +1603,12 @@ redmond_draw_handle (GtkStyle * style,
              gint height, 
              GtkOrientation orientation)
 {
+  RedmondStyle *redmond_style = REDMOND_STYLE (style);
+
   gboolean left_cutoff = FALSE, right_cutoff = FALSE, top_cutoff = FALSE, bottom_cutoff = FALSE;
 
-  g_return_if_fail(sanitize_parameters(style, window, &width, &height));
+  CHECK_ARGS
+  SANITIZE_SIZE
   
   if (IS_BONOBO_DOCK_ITEM_GRIP(widget) && 
      (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_RTL) && 
@@ -1654,30 +1666,29 @@ redmond_draw_handle (GtkStyle * style,
 	      if (width < height)
 		{
 		  orientation = GTK_ORIENTATION_HORIZONTAL;
-		  if (is_panel_widget_item (widget))
+		  if (ge_is_panel_widget_item (widget))
 		    x += 2;
 		}
 	      else
 		{
 		  orientation = GTK_ORIENTATION_VERTICAL;
-		  if (is_panel_widget_item (widget))
+		  if (ge_is_panel_widget_item (widget))
 		    y += 2;
 		}
 	    }
 	}
+
+      cairo_t *cr = ge_gdk_drawable_to_cairo (window, area);
  
       /* draw the drag bar */
       if (orientation == GTK_ORIENTATION_VERTICAL)
         {
-           cairo_t *cr = redmond_begin_paint (window, area);
           
-	  do_redmond_draw_shadow (cr, &style->light[state_type],
-			          &style->dark[state_type],
+	  ge_cairo_simple_border (cr, &redmond_style->color_cube.light[state_type],
+			          &redmond_style->color_cube.dark[state_type],
 			          x + style->xthickness + 1, y + height / 2 - 1,
 			          width - style->xthickness - 3, 3, FALSE);
 
-            cairo_destroy(cr);
- 	
           bottom_cutoff = TRUE;
         }
       else
@@ -1685,16 +1696,13 @@ redmond_draw_handle (GtkStyle * style,
           right_cutoff = (!widget) || (gtk_widget_get_direction (widget) == GTK_TEXT_DIR_LTR);
           left_cutoff = !right_cutoff;
  
-           cairo_t *cr = redmond_begin_paint (window, area);
-          
-	  do_redmond_draw_shadow (cr, &style->light[state_type],
-			          &style->dark[state_type], x + width / 2 - 1,
+	  ge_cairo_simple_border (cr, &redmond_style->color_cube.light[state_type],
+			          &redmond_style->color_cube.dark[state_type], x + width / 2 - 1,
 			          y + style->ythickness + 1, 3,
 			          height - style->ythickness - 3, FALSE);
-
-           cairo_destroy(cr);			          
         }
-      if (is_panel_widget_item (widget)
+
+      if (ge_is_panel_widget_item (widget)
 	  && (CHECK_DETAIL (detail, "handlebox")
 	      && (!IS_HANDLE_BOX_ITEM (widget)))
 	  && (!(IS_HANDLE_BOX (widget))))
@@ -1704,16 +1712,16 @@ redmond_draw_handle (GtkStyle * style,
 	   */
 	  if (orientation == GTK_ORIENTATION_VERTICAL)
 	    {
-	      gdk_draw_line (window, style->dark_gc[state_type], x + 1, y + 0,
+	      ge_cairo_line (cr, &redmond_style->color_cube.dark[state_type], x + 1, y + 0,
 			     x + width - 2, y + 0);
-	      gdk_draw_line (window, style->light_gc[state_type], x + 1,
+	      ge_cairo_line (cr, &redmond_style->color_cube.light[state_type], x + 1,
 			     y + 1, x + width - 2, y + 1);
 	    }
 	  else
 	    {
-	      gdk_draw_line (window, style->dark_gc[state_type], x + 0, y + 1,
+	      ge_cairo_line (cr, &redmond_style->color_cube.dark[state_type], x + 0, y + 1,
 			     x + 0, y + height - 2);
-	      gdk_draw_line (window, style->light_gc[state_type], x + 1,
+	      ge_cairo_line (cr, &redmond_style->color_cube.light[state_type], x + 1,
 			     y + 1, x + 1, y + height - 2);
 	    }
 	}
@@ -1756,17 +1764,15 @@ redmond_draw_handle (GtkStyle * style,
              
           if (!skip_shadow)
             {
-                cairo_t *cr = redmond_begin_paint (window, &clip);
-          
-   	       do_redmond_draw_shadow (cr, &style->light[state_type],
-			               &style->dark[state_type],
+   	       ge_cairo_simple_border (cr, &redmond_style->color_cube.light[state_type],
+			               &redmond_style->color_cube.dark[state_type],
                                        x - 2*left_cutoff, 
                                        y - 2*top_cutoff, 
                                        width + 2*left_cutoff + 2*right_cutoff,
 			               height + 2*top_cutoff + 2*bottom_cutoff,
 			               FALSE);
-              cairo_destroy(cr);			          
             }
 	}
+           cairo_destroy(cr);			          
     }
 }
